cmake_minimum_required(VERSION 3.16..3.25)

set(adplug_known_comps static shared)
set(adplug_comp_static NO)
set(adplug_comp_shared NO)
foreach (adplug_comp IN LISTS ${CMAKE_FIND_PACKAGE_NAME}_FIND_COMPONENTS)
    if (adplug_comp IN_LIST adplug_known_comps)
        set(adplug_comp_${adplug_comp} YES)
    else ()
        set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
            "adplug does not recognize component `${adplug_comp}`.")
        set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
        return()
    endif ()
endforeach ()

if (adplug_comp_static AND adplug_comp_shared)
    set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
        "adplug `static` and `shared` components are mutually exclusive.")
    set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
    return()
endif ()

set(adplug_static_targets "${CMAKE_CURRENT_LIST_DIR}/adplug-static-targets.cmake")
set(adplug_shared_targets "${CMAKE_CURRENT_LIST_DIR}/adplug-shared-targets.cmake")

macro(adplug_load_targets type)
    if (NOT EXISTS "${adplug_${type}_targets}")
        set(${CMAKE_FIND_PACKAGE_NAME}_NOT_FOUND_MESSAGE
            "adplug `${type}` libraries were requested but not found.")
        set(${CMAKE_FIND_PACKAGE_NAME}_FOUND FALSE)
        return()
    endif ()
    include("${adplug_${type}_targets}")
endmacro()

if (adplug_comp_static)
    adplug_load_targets(static)
elseif (adplug_comp_shared)
    adplug_load_targets(shared)
elseif (DEFINED adplug_SHARED_LIBS AND adplug_SHARED_LIBS)
    adplug_load_targets(shared)
elseif (DEFINED adplug_SHARED_LIBS AND NOT adplug_SHARED_LIBS)
    adplug_load_targets(static)
elseif (BUILD_SHARED_LIBS)
    if (EXISTS "${adplug_shared_targets}")
        adplug_load_targets(shared)
    else ()
        adplug_load_targets(static)
    endif ()
else ()
    if (EXISTS "${adplug_static_targets}")
        adplug_load_targets(static)
    else ()
        adplug_load_targets(shared)
    endif ()
endif ()
